# coding=utf-8
#
# Copyright (c) 2017 Aliyun.com All right reserved. This software is the
# confidential and proprietary information of Aliyun.com ("Confidential
# Information"). You shall not disclose such Confidential Information and shall
# use it only in accordance with the terms of the license agreement you entered
# into with Aliyun.com .
#
# created by quming on 07/18/2017

from aliyunsdkros.request.v20150901 import DescribeStacksRequest

import ros.apps.config as connect
import ros.apps.utils as utils
import json
import sys
import math

reload(sys)
sys.setdefaultencoding('utf-8')


def setup(subparsers):
    parser = subparsers.add_parser('list-stacks', help='Returns the summary information for stacks whose status matches the specified StackStatusFilter')
    parser.add_argument('--stack-name', help='The name that is associated with the stack', default=None)
    parser.add_argument('--stack-id', help='The id that is associated with the stack', default=None)
    parser.add_argument('--status', help='status of stacks', choices=['CREATE_COMPLETE', 'CREATE_FAILED', 'CREATE_IN_PROGRESS',\
     'DELETE_COMPLETE', 'DELETE_FAILED', 'DELETE_IN_PROGRESS', 'ROLLBACK_COMPLETE', 'ROLLBACK_FAILED', 'ROLLBACK_IN_PROGRESS'], default=None)
    parser.add_argument('--region-id', help='The region of stacks')
    parser.add_argument('--page-number', help='The page number of stack lists, start from 1, default 1', type=int, default=1)
    parser.add_argument('--page-size', help='Lines each page, max 100, default 10', type=int, default=10)

    parser.set_defaults(func=list_stacks)


def list_stacks(args):
    req = prepare_request(args)
    status, headers, body = utils.send_req(req)
    utils.deal_resp(status, headers, body, print_response)


def prepare_request(args):
    req = DescribeStacksRequest.DescribeStacksRequest()
    # No function in DescribeStacksRequest to set region-id

    if args.region_id is not None:
        req.set_headers({'x-acs-region-id': args.region_id})
    else:
        req.set_headers({'x-acs-region-id': connect.REGION_ID})

    if args.stack_name is not None:
        req.set_Name(args.stack_name)

    if args.stack_id is not None:
        req.set_StackId(args.stack_id)

    if args.status is not None:
        req.set_Status(args.stack_status)

    req.set_PageNumber(args.page_number)
    req.set_PageSize(args.page_size)

    return req


def print_response(data):

    print('\nTotal Records: %d     Page: %d/%d' % (data.get('TotalCount'), data.get('PageNumber'),
            math.ceil(float(data.get('TotalCount'))/data.get('PageSize'))))

    if connect.JSON_FORM:
        jsonDumpsIndentStr = json.dumps(data, indent=connect.JSON_INDENT, ensure_ascii=False, sort_keys=True)
        print(jsonDumpsIndentStr)
    else:
        for item in data.get('Stacks'):
            print('\n%-20s:  %s' % ('Id', item.get('Id')))
            print('%-20s:  %s' % ('Name', item.get('Name')))
            print('%-20s:  %s' % ('Description', item.get('Description')))
            print('%-20s:  %s' % ('Region', item.get('Region')))
            print('%-20s:  %s' % ('Status', item.get('Status')))
            print('%-20s:  %s' % ('StatusReason', item.get('StatusReason')))
            print('%-20s:  %s' % ('TimeoutMins', item.get('TimeoutMins')))
            print('%-20s:  %s' % ('DisableRollback', item.get('DisableRollback')))
            print('%-20s:  %s' % ('Created', item.get('Created')))
            print('%-20s:  %s\n' % ('Updated', item.get('Updated')))



# coding=utf-8
#
# Copyright (c) 2017 Aliyun.com All right reserved. This software is the
# confidential and proprietary information of Aliyun.com ("Confidential
# Information"). You shall not disclose such Confidential Information and shall
# use it only in accordance with the terms of the license agreement you entered
# into with Aliyun.com .
#
# created by quming on 07/18/2017

from aliyunsdkcore.client import AcsClient
from aliyunsdkcore.acs_exception.exceptions import ClientException
from aliyunsdkcore.acs_exception.exceptions import ServerException

import NewConfigParser
import os
import sys
import re

reload(sys)
sys.setdefaultencoding('utf-8')

ACCESS_KEY_ID = None
ACCESS_KEY_SECRET = None
REGION_ID = None
client = None
JSON_FORM = False
JSON_INDENT = 2
ROS_DEBUG = False


def current_conf():
    """
    Print current client configuration
    :return: None
    """
    global ACCESS_KEY_ID
    global ACCESS_KEY_SECRET
    global REGION_ID

    print(
        "[DEBUG] Current Config:\nACCESS_KEY_ID: %s\nACCESS_KEY_SECRET: %s\nREGION_ID: %s\n" %
        (ACCESS_KEY_ID, ACCESS_KEY_SECRET, REGION_ID))


def set_client(cfg_file, region_id, top_dir=None):
    """
    Configure client
    :param cfg_file: specify the configuration file
    :param region_id: specify region id
    :param top_dir: working path
    :return: None
    """
    global ACCESS_KEY_ID
    global ACCESS_KEY_SECRET
    global REGION_ID
    global client
    global JSON_INDENT
    global ROS_DEBUG

    if top_dir is None:
        default_file = 'ros/ros.conf'
    else:
        default_file = os.path.normpath(top_dir + '/ros/ros.conf')

    cf = NewConfigParser.NewConfigParser()
    if cfg_file is None:
        if ROS_DEBUG:
            print("Use default config file: %s\n" % default_file)
        cfg_file = default_file

    if os.path.isfile(cfg_file):
        pass
    else:
        if os.path.isdir(top_dir + '/ros'):
            pass
        else:
            os.mkdir(top_dir + '/ros')
            
        print('Please set Aliyun access info first.')

        access_key_id = raw_input('Enter your access key id:')
        while check_access_info(access_key_id) is False:
            access_key_id = raw_input('Enter your access key id, only characters and numbers:')
        
        access_key_secret = raw_input('Enter your access key secret, without quote:')
        while check_access_info(access_key_secret) is False:
            access_key_secret = raw_input('Enter your access key secret, only characters and numbers:')

        default_region_id = raw_input('Enter default region id, without quote:')

        cf.add_section('ACCESS')
        cf.set('ACCESS', 'ACCESS_KEY_ID', access_key_id)
        cf.set('ACCESS', 'ACCESS_KEY_SECRET', access_key_secret)
        cf.set('ACCESS', 'REGION_ID', default_region_id)

        cf.add_section('OTHER')
        cf.set('OTHER', 'JSON_INDENT', 2)
        cf.set('OTHER', 'DEBUG', False)

        with open(cfg_file, 'w') as configfile:
            cf.write(configfile)

    try:
        cf.read(cfg_file)
    except BaseException:
        print("""Config file (%s) error, please write it like:

        [ACCESS]
        ACCESS_KEY_ID = YOUR_KEY_ID
        ACCESS_KEY_SECRET = YOUR_KEY_SECRET
        REGION_ID = cn-beijing

        [OTHER]
        JSON_INDENT = 2
        DEBUG = False
        """ % cfg_file)
        sys.exit(1)

    ACCESS_KEY_ID = cf.get("ACCESS", "ACCESS_KEY_ID")
    ACCESS_KEY_SECRET = cf.get("ACCESS", "ACCESS_KEY_SECRET")
    if region_id is None:
        REGION_ID = cf.get("ACCESS", "REGION_ID")
    else:
        REGION_ID = region_id

    JSON_INDENT = int(cf.get("OTHER", "JSON_INDENT"))
    ROS_DEBUG = bool(cf.get("OTHER", "DEBUG") == 'True')

    client = AcsClient(
        ACCESS_KEY_ID,
        ACCESS_KEY_SECRET,
        REGION_ID
    )

    if ROS_DEBUG:
        current_conf()


def check_access_info(info):
    """
    Check if access info only has characters and numbers
    """
    match = re.search('^[A-Za-z0-9]+$', info)

    if match:
        return True
    else:
        return False



# coding=utf-8
#
# Copyright (c) 2017 Aliyun.com All right reserved. This software is the
# confidential and proprietary information of Aliyun.com ("Confidential
# Information"). You shall not disclose such Confidential Information and shall
# use it only in accordance with the terms of the license agreement you entered
# into with Aliyun.com .
#
# created by quming on 07/18/2017

import sys
import os
import ros.apps.NewConfigParser as NewConfigParser
import ros.apps.config as connect

reload(sys)
sys.setdefaultencoding('utf-8')

POSSIBLE_TOPDIR = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]), os.pardir, os.pardir))


def setup(subparsers):
    parser = subparsers.add_parser('set-userdata', help='Set default Aliyun access info')
    parser.add_argument('--key-id', help='The default Aliyun access key id', required=True)
    parser.add_argument('--key-secret', help='The default Aliyun access key region', required=True)
    parser.add_argument('--region-id', help='The default region', required=True)
    parser.add_argument('--json-ident', help='The default json indent when output in json format', default=2)
    parser.add_argument('--debug', help='Whether to read debug infos', choices=['False', 'True'], default='False')

    parser.set_defaults(func=set_userdata)


def set_userdata(args):

    default_file = os.path.normpath(POSSIBLE_TOPDIR + '/ros/ros.conf')

    if connect.check_access_info(args.key_id) is False or \
        connect.check_access_info(args.key_secret) is False:
        print('Please check your accrss key id and secret, should only have characters and numbers.')
        sys.exit(1)

    cf = NewConfigParser.NewConfigParser()
    cf.add_section('ACCESS')
    cf.set('ACCESS', 'ACCESS_KEY_ID', args.key_id)
    cf.set('ACCESS', 'ACCESS_KEY_SECRET', args.key_secret)
    cf.set('ACCESS', 'REGION_ID', args.region_id)

    cf.add_section('OTHER')
    cf.set('OTHER', 'JSON_INDENT', args.json_ident)
    cf.set('OTHER', 'DEBUG', args.debug)

    try:
        with open(default_file, 'wb') as configfile:
            cf.write(configfile)
        print('Write : %s' % default_file)
    except Exception, e:
        print('Something wrong:\n%s' % str(e))
        sys.exit(1)



# coding=utf-8
#
# Copyright (c) 2017 Aliyun.com All right reserved. This software is the
# confidential and proprietary information of Aliyun.com ("Confidential
# Information"). You shall not disclose such Confidential Information and shall
# use it only in accordance with the terms of the license agreement you entered
# into with Aliyun.com .
#
# created by quming on 07/18/2017

import ConfigParser


class NewConfigParser(ConfigParser.ConfigParser):
    '''
    Make options keep upper case
    '''
    def __init__(self,defaults=None):
        ConfigParser.ConfigParser.__init__(self, defaults=None)

    def optionxform(self, optionstr):
        return optionstr



# coding=utf-8
#
# Copyright (c) 2017 Aliyun.com All right reserved. This software is the
# confidential and proprietary information of Aliyun.com ("Confidential
# Information"). You shall not disclose such Confidential Information and shall
# use it only in accordance with the terms of the license agreement you entered
# into with Aliyun.com .
#
# created by quming on 07/18/2017

from aliyunsdkros.request.v20150901 import PreviewStackRequest

import ros.apps.config as connect
import ros.apps.utils as utils
import json
import sys

reload(sys)
sys.setdefaultencoding('utf-8')


def setup(subparsers):
    # create a parser for the 'create-stack' command
    parser = subparsers.add_parser('preview-stack', help='Preview a stack as specified in the template')
    parser.add_argument('--region-id', help='The region that is associated with the stack')
    parser.add_argument('--stack-name', help='The name that is associated with the stack', required=True)
    parser.add_argument('--template-url', help='Location of file containing the template body', required=True)
    parser.add_argument('--parameters', help='A list of Parameter structures that specify input parameters for the stack. Synatax: key=value,key=value')
    parser.add_argument('--disable-rollback', help='Set to true to disable rollback of the stack if stack creation failed', default=True, type=bool)
    parser.add_argument('--timeout-in-minutes', help='The amount of time that can pass before the stack status becomes CREATE_FAILED', default=60, type=int)

    parser.set_defaults(func=preview_stack)


def preview_stack(args):
    req = prepare_request(args)
    status, headers, body = utils.send_req(req)
    utils.deal_resp(status, headers, body, print_response)


def prepare_request(args):
    req = PreviewStackRequest.PreviewStackRequest()

    if args.region_id is not None:
        req.set_headers({'x-acs-region-id': args.region_id})
    else:
        req.set_headers({'x-acs-region-id': connect.REGION_ID})

    content = {}
    content['Name'] = args.stack_name

    file_context = utils.read_template(args.template_url)
    content['Template'] = file_context

    content['DisableRollback'] = args.disable_rollback
    content['TimeoutMins'] = args.timeout_in_minutes

    ps = {}
    if args.parameters is not None:
        s = args.parameters.split(',')
        for item in s:
            pair = item.split('=')
            ps[pair[0]] = pair[1]

    content['Parameters'] = ps

    jsonDumpsIndentStr = json.dumps(content, indent=connect.JSON_INDENT, ensure_ascii=False, sort_keys=True)
    # print(jsonDumpsIndentStr)
    req.set_content(jsonDumpsIndentStr)

    return req


def print_response(data):
    if connect.JSON_FORM:
        jsonDumpsIndentStr = json.dumps(data, indent=connect.JSON_INDENT, ensure_ascii=False, sort_keys=True)
        print(jsonDumpsIndentStr)
    else:
        print("%-20s:  %s" % ('Id', data.get("Id")))
        print("%-20s:  %s" % ('Name', data.get("Name")))
        print("%-20s:  %s" % ('Description', data.get("Description")))
        print("%-20s:  %s" % ('Region', data.get("Region")))
        print("%-20s:  %s" % ('DisableRollback', data.get("DisableRollback")))
        print("%-20s:  %s" % ('TimeoutMins', data.get("TimeoutMins")))
        print("%-20s:  %s" % ('Created', data.get("Created")))
        print("%-20s:  %s" % ('Updated', data.get("Updated")))
        print("%-20s:  %s" % ('Webhook', data.get("Webhook")))
        print("%-20s:  %s" % ('TemplateDescription', data.get("TemplateDescription")))

        print("\nParameters:")
        for (k,v) in data.get("Parameters").items():
            print("    %-20s: %s" % (k,v))

        print("\nResources:")
        for resource in data.get("Resources"):
            print("\n-----------------------------------------------------------------------")
            print("\nResource:\n")
            print("    %-20s:  %s" % ('StackName', data.get("StackName")))
            print("    %-20s:  %s" % ('ResourceType', data.get("ResourceType")))
            print("    %-20s:  %s" % ('ResourceName', data.get("ResourceName")))
            print("    %-20s:  %s" % ('ResourceStatus', data.get("ResourceStatus")))
            print("    %-20s:  %s" % ('ResourceStatusReason', data.get("ResourceStatusReason")))
            print("    %-20s:  %s" % ('ResourceData', data.get("ResourceData")))
            print("    %-20s:  %s" % ('Description', data.get("Description")))
            print("    %-20s:  %s" % ('ResourceAction', data.get("ResourceAction")))
            print("    %-20s:  %s" % ('PhysicalResourceId', data.get("PhysicalResourceId")))
            print("    %-20s:  %s" % ('CreatedTime', data.get("CreatedTime")))
            print("    %-20s:  %s" % ('UpdatedTime', data.get("UpdatedTime")))
            print("    %-20s:  %s" % ('DeletedTime', data.get("DeletedTime")))

            print("\n    RequiredBy:")
            for key in resource.get("RequiredBy"):
                print("        %s" % key)

            print("\n    Attributes:")
            for (k,v) in resource.get("Attributes").items():
                print("        %-20s:  %s" % (k,v))
            print("\n    Metadata:")
            for (k,v) in resource.get("Metadata").items():
                print("        %-20s:  %s" % (k,v))
            print("\n    Properties:")
            for (k,v) in resource.get("Properties").items():
                print("        %-20s:  %s" % (k,v))
            print("\n    ResourceIdentity:")
            for (k,v) in resource.get("ResourceIdentity").items():
                print("        %-20s:  %s" % (k,v))
            print("\n    StackIdentity:")
            for (k,v) in resource.get("StackIdentity").items():
                print("        %-20s:  %s" % (k,v))



# coding=utf-8
#
# Copyright (c) 2017 Aliyun.com All right reserved. This software is the
# confidential and proprietary information of Aliyun.com ("Confidential
# Information"). You shall not disclose such Confidential Information and shall
# use it only in accordance with the terms of the license agreement you entered
# into with Aliyun.com .
#
# created by quming on 07/18/2017

from aliyunsdkros.request.v20150901 import UpdateStackRequest

import ros.apps.config as connect
import ros.apps.utils as utils
import json
import sys

reload(sys)
sys.setdefaultencoding('utf-8')


def setup(subparsers):
    # create a parser for the 'create-stack' command
    parser = subparsers.add_parser('update-stack', help='Update a stack as specified in the template')
    parser.add_argument('--region-id', help='The region that is associated with the stack', required=True)
    parser.add_argument('--stack-name', help='The name that is associated with the stack', required=True)
    parser.add_argument('--stack-id', help='The id that is associated with the stack', required=True)
    parser.add_argument('--template-url', help='Location of file containing the template body', required=True)
    parser.add_argument('--parameters', help='A list of Parameter structures that specify input parameters for the stack. Synatax: key=value,key=value')
    parser.add_argument('--disable-rollback', help='Set to true to disable rollback of the stack if stack creation failed', default=True, type=bool)
    parser.add_argument('--timeout-in-minutes', help='The amount of time that can pass before the stack status becomes CREATE_FAILED', default=60, type=int)

    parser.set_defaults(func=update_stack)


def update_stack(args):
    req = prepare_request(args)
    status, headers, body = utils.send_req(req)
    utils.deal_resp(status, headers, body, print_response)


def prepare_request(args):
    req = UpdateStackRequest.UpdateStackRequest()

    req.set_headers({'x-acs-region-id': args.region_id})
    req.set_StackName(args.stack_name)
    req.set_StackId(args.stack_id)
    
    content = {}

    file_context = utils.read_template(args.template_url)
    content['Template'] = file_context

    content['DisableRollback'] = args.disable_rollback
    content['TimeoutMins'] = args.timeout_in_minutes

    ps = {}
    if args.parameters is not None:
        s = args.parameters.split(',')
        for item in s:
            pair = item.split('=')
            ps[pair[0]] = pair[1]

    content['Parameters'] = ps

    jsonDumpsIndentStr = json.dumps(content, indent=connect.JSON_INDENT, ensure_ascii=False, sort_keys=True)
    # print(jsonDumpsIndentStr)
    req.set_content(jsonDumpsIndentStr)

    return req


def print_response(data):
    if connect.JSON_FORM:
        jsonDumpsIndentStr = json.dumps(data, indent=connect.JSON_INDENT, ensure_ascii=False, sort_keys=True)
        print(jsonDumpsIndentStr)
    else:
        for (k, v) in data.items():
            print('%-20s:  %s' % (k, v))




# coding=utf-8
#
# Copyright (c) 2017 Aliyun.com All right reserved. This software is the
# confidential and proprietary information of Aliyun.com ("Confidential
# Information"). You shall not disclose such Confidential Information and shall
# use it only in accordance with the terms of the license agreement you entered
# into with Aliyun.com .
#
# created by quming on 07/18/2017

from aliyunsdkcore.acs_exception.exceptions import ClientException
from aliyunsdkcore.acs_exception import error_code, error_msg
import ros.apps.config as connect
import sys
import json
import urllib2

reload(sys)
sys.setdefaultencoding('utf-8')

def print_error(data):
    """
    Output error response from aliyun server
    :param data: response body in json
    :return: None
    """

    for (k, v) in data.items():
        print('%-20s:  %s' % (k, v))


def read_template(template_url):
    """
    Get template content, support local file and online url
    :param template_url: the url of the template
    :return: template content
    """
    if template_url.startswith('http'):
        try:
            response = urllib2.urlopen(template_url)
            return response.read()
        except Exception as e:
            print('Something wrong:\n%s' % str(e))
            sys.exit(1)
    else:
        try:
            with open(template_url, 'r') as file_object:
                file_context = file_object.read()
            return file_context
        except Exception as e:
            print('Something wrong:\n%s' % str(e))
            sys.exit(1)


def alignment(s, space, align='left'):
    """
    In python 2.x, make Chinese characters keep align
    :param str: input str
    :param space: width of the str
    :param align: left\right\center
    :return: aligned str
    """
    length = len(s.encode('gb2312'))
    space = space - length if space >= length else 0
    if align == 'left':
        s = s + ' ' * space
    elif align == 'right':
        s = ' ' * space + s
    elif align == 'center':
        s = ' ' * (space // 2) + s + ' ' * (space - space // 2)
    return s


def send_req(req):
    """
    Send ros request
    :param req: request
    :return: None
    """
    req.set_accept_format("JSON")
    req.set_content_type("application/json;charset=utf-8")

    if connect.ROS_DEBUG:
        print('[DEBUG] Send request:\n %s\n' % req)

    try:
        status, headers, body = get_raw_resp(req)
    except Exception, e:
        print('Something wrong:\n%s' % str(e))
        sys.exit(1)

    return status, headers, body


def deal_resp(status, headers, body, print_response):
    """
    Output response
    :param status: status code
    :param headers: response header
    :param body: response body
    :param print_response: print function
    :return: None
    """

    if connect.ROS_DEBUG:
        print('[DEBUG] Response status:\n %s\n' % status)
        print('[DEBUG] Response headers:\n %s\n' % headers)
        print('[DEBUG] Response body:\n %s\n' % body)

    try:
        data = json.loads(body) if body != "" else ("Response status: %s" % (status))

        if type(data) == dict:
            if 200 <= status < 300:
                print("[Succeed]")
                print_response(data)
            else:
                print("[Failed]")
                print_error(data)
        else:
            print data
    except Exception, e:
        print("[Error]")
        print('Something wrong:\n%s' % str(e))
        sys.exit(1)


def get_raw_resp(request):
    """
    Get RAW response of aliyunsdk
    :param client: aliyunsdk client
    :param request: request to send
    :return: None
    """

    client = connect.client
    endpoint = client._resolve_endpoint(request)
    http_response = client._make_http_response(endpoint, request)
    if client._url_test_flag:
        raise ClientException("URLTestFlagIsSet", http_response.get_url())

    # Do the actual network thing
    try:
        status, headers, body = http_response.get_response_object()
        return status, headers, body
    except IOError as e:
        raise ClientException(
            error_code.SDK_SERVER_UNREACHABLE,
            error_msg.get_msg('SDK_SERVER_UNREACHABLE') + ': ' + str(e))
    except AttributeError:
        raise ClientException(
            error_code.SDK_INVALID_REQUEST,
            error_msg.get_msg('SDK_INVALID_REQUEST'))


def recursively_print(data, flag=True, indent=0):
    """
    Print dict\list recursively
    :param data: data to print
    :param flag: whether need to use indentation
    :param indent: passed father level's indent
    :return: None
    """
    if isinstance(data, list):
        for item in data:
            print('')
            recursively_print(item, False, indent + 1)

    elif isinstance(data, dict):
        for (k, v) in data.items():
            if indent == 0:
                print '\n\n===================================================\n',
            elif indent == 1:
                print '\n\n    -----------------------------------------------\n',
            print('')
            recursively_print(k, False, indent)
            print':  ',
            recursively_print(v, True, indent + 1)

    else:
        data_out = str(data)
        pre = ''
        if not flag:
            pre = '    '
        for i in range(0, indent):
            data_out = pre + data_out
        print data_out,


if __name__ == '__main__':
    resp = read_template('http://ros-template.cn-hangzhou.oss.aliyun-inc.com/ecs_vpc_instance.json')
    print(resp)


# coding=utf-8
#
# Copyright (c) 2017 Aliyun.com All right reserved. This software is the
# confidential and proprietary information of Aliyun.com ("Confidential
# Information"). You shall not disclose such Confidential Information and shall
# use it only in accordance with the terms of the license agreement you entered
# into with Aliyun.com .
#
# created by quming on 07/18/2017

from aliyunsdkros.request.v20150901 import DescribeStackDetailRequest

import ros.apps.config as connect
import ros.apps.utils as utils
import json
import sys

reload(sys)
sys.setdefaultencoding('utf-8')


def setup(subparsers):
    parser = subparsers.add_parser('describe-stack', help='Returns the description for the specified stack')
    parser.add_argument('--stack-name', help='The name that is associated with the stack', required=True)
    parser.add_argument('--stack-id', help='The id that is associated with the stack', required=True)

    parser.set_defaults(func=describe_stack)


def describe_stack(args):
    req = prepare_request(args)
    status, headers, body = utils.send_req(req)
    utils.deal_resp(status, headers, body, print_response)


def prepare_request(args):
    req = DescribeStackDetailRequest.DescribeStackDetailRequest()
    req.set_headers({'x-acs-region-id': connect.REGION_ID})
    req.set_StackName(args.stack_name)
    req.set_StackId(args.stack_id)

    return req


def print_response(data):
    if(connect.JSON_FORM):
        jsonDumpsIndentStr = json.dumps(data, indent=connect.JSON_INDENT, ensure_ascii=False, sort_keys=True)
        print(jsonDumpsIndentStr)
    else:
        print("%-20s:  %s" % ('Name', data.get("Name")))
        print("%-20s:  %s" % ('Id', data.get("Id")))
        print("%-20s:  %s" % ('Description', data.get("Description")))
        print("%-20s:  %s" % ('Region', data.get("Region")))
        print("%-20s:  %s" % ('Status', data.get("Status")))
        print("%-20s:  %s" % ('StatusReason', data.get("StatusReason")))
        print("%-20s:  %s" % ('DisableRollback', data.get("DisableRollback")))
        print("%-20s:  %s" % ('TimeoutMins', data.get("TimeoutMins")))
        print("%-20s:  %s" % ('Created', data.get("Created")))
        print("%-20s:  %s" % ('Updated', data.get("Updated")))
        print("%-20s:  %s" % ('Webhook', data.get("Webhook")))

        print("\nParameters:")
        if data.get("Parameters") is not None:
            for (k,v) in data.get("Parameters").items():
                print("    %-20s: %s" % (k,v))

        print("\nOutputs:")
        if data.get("Outputs") is not None:
            for out in data.get("Outputs"):
                print("    %-20s: %s --- %s" % (out["OutputKey"], out["OutputValue"], out["Description"]))
    


# coding=utf-8
#
# Copyright (c) 2017 Aliyun.com All right reserved. This software is the
# confidential and proprietary information of Aliyun.com ("Confidential
# Information"). You shall not disclose such Confidential Information and shall
# use it only in accordance with the terms of the license agreement you entered
# into with Aliyun.com .
#
# created by quming on 07/18/2017

from aliyunsdkros.request.v20150901 import AbandonStackRequest

import ros.apps.config as connect
import ros.apps.utils as utils
import json
import sys

reload(sys)
sys.setdefaultencoding('utf-8')


def setup(subparsers):
    parser = subparsers.add_parser('abandon-stack', help='Abandon the specified stack')
    parser.add_argument('--region-id', help='The region that is associated with the stack', required=True)
    parser.add_argument('--stack-name', help='The name that is associated with the stack', required=True)
    parser.add_argument('--stack-id', help='The id that is associated with the stack', required=True)

    parser.set_defaults(func=abandon_stack)


def abandon_stack(args):
    req = prepare_request(args)
    status, headers, body = utils.send_req(req)
    utils.deal_resp(status, headers, body, print_response)


def prepare_request(args):
    req = AbandonStackRequest.AbandonStackRequest()

    req.set_headers({'x-acs-region-id': args.region_id})
    req.set_StackName(args.stack_name)
    req.set_StackId(args.stack_id)

    return req


def print_response(data):
    #if connect.JSON_FORM:
    jsonDumpsIndentStr = json.dumps(data, indent=connect.JSON_INDENT, ensure_ascii=False, sort_keys=True)
    print(jsonDumpsIndentStr)


# coding=utf-8
#
# Copyright (c) 2017 Aliyun.com All right reserved. This software is the
# confidential and proprietary information of Aliyun.com ("Confidential
# Information"). You shall not disclose such Confidential Information and shall
# use it only in accordance with the terms of the license agreement you entered
# into with Aliyun.com .
#
# created by quming on 07/18/2017

from aliyunsdkros.request.v20150901 import DescribeEventsRequest

import ros.apps.config as connect
import ros.apps.utils as utils
import json
import sys
import math

reload(sys)
sys.setdefaultencoding('utf-8')


def setup(subparsers):
    parser = subparsers.add_parser('list-events', help='Returns all stack related events for a specified stack in reverse chronological order')
    parser.add_argument('--stack-name', help='The name that is associated with the stack', required=True)
    parser.add_argument('--stack-id', help='The id that is associated with the stack', required=True)
    parser.add_argument('--resource-status', help='status of resources: COMPLETE\FAILED\IN_PROGRESS', choices=['COMPLETE', 'FAILED', 'IN_PROGRESS'])
    parser.add_argument('--resource-name', help='The name of resources')
    parser.add_argument('--resource-type', help='The type of resources')
    parser.add_argument('--page-number', help='The page number of stack lists, start from 1, default 1', type=int, default=1)
    parser.add_argument('--page-size', help='Lines each page, max 100, default 10', type=int, default=10)
    
    parser.set_defaults(func=list_events)


def list_events(args):
    req = prepare_request(args)
    status, headers, body = utils.send_req(req)
    utils.deal_resp(status, headers, body, print_response)


def prepare_request(args):
    req = DescribeEventsRequest.DescribeEventsRequest()
    req.set_StackName(args.stack_name)
    req.set_StackId(args.stack_id)

    if args.resource_status is not None:
        req.set_ResourceStatus(args.resource_status)

    if args.resource_name is not None:
        req.set_ResourceName(args.resource_name)

    if args.resource_type is not None:
        req.set_ResourceType(args.resource_type)

    req.set_PageNumber(args.page_number)
    req.set_PageSize(args.page_size)

    return req


def print_response(data):
    if connect.JSON_FORM:
        jsonDumpsIndentStr = json.dumps(data, indent=connect.JSON_INDENT, ensure_ascii=False, sort_keys=True)
        print(jsonDumpsIndentStr)
    else:
        print('\nTotal Records: %d     Page: %d/%d' % (data.get('TotalCount'), data.get('PageNumber'),
            math.ceil(float(data.get('TotalCount'))/data.get('PageSize'))))

        for item in data.get('Events'):
            print('\n%-20s:  %s' % ('Time', item.get('Time')))
            print('%-20s:  %s' % ('ResourceName', item.get('ResourceName')))
            print('%-20s:  %s' % ('ResourceType', item.get('ResourceType')))
            print('%-20s:  %s' % ('Type', item.get('Type')))
            print('%-20s:  %s' % ('Status', item.get('Status')))
            print('%-20s:  %s\n' % ('StatusReason', item.get('StatusReason')))



# coding=utf-8
#
# Copyright (c) 2017 Aliyun.com All right reserved. This software is the
# confidential and proprietary information of Aliyun.com ("Confidential
# Information"). You shall not disclose such Confidential Information and shall
# use it only in accordance with the terms of the license agreement you entered
# into with Aliyun.com .
#
# created by quming on 07/18/2017

from aliyunsdkros.request.v20150901 import DescribeTemplateRequest

import ros.apps.config as connect
import ros.apps.utils as utils
import json
import sys

reload(sys)
sys.setdefaultencoding('utf-8')


def setup(subparsers):
    parser = subparsers.add_parser('get-template', help='Returns the template body for a specified stack')
    parser.add_argument('--stack-name', help='The name that is associated with the stack', required=True)
    parser.add_argument('--stack-id', help='The id that is associated with the stack', required=True)

    parser.set_defaults(func=get_template)


def get_template(args):
    req = prepare_request(args)
    status, headers, body = utils.send_req(req)
    utils.deal_resp(status, headers, body, print_response)


def prepare_request(args):
    req = DescribeTemplateRequest.DescribeTemplateRequest()
    req.set_StackName(args.stack_name)
    req.set_StackId(args.stack_id)

    return req


def print_response(data):
    # if connect.JSON_FORM:
    jsonDumpsIndentStr = json.dumps(data, indent=connect.JSON_INDENT, ensure_ascii=False, sort_keys=True)
    print(jsonDumpsIndentStr)
    # else:
    #      utils.recursivePrint(list)




# coding=utf-8
#
# Copyright (c) 2017 Aliyun.com All right reserved. This software is the
# confidential and proprietary information of Aliyun.com ("Confidential
# Information"). You shall not disclose such Confidential Information and shall
# use it only in accordance with the terms of the license agreement you entered
# into with Aliyun.com .
#
# created by quming on 07/18/2017

from aliyunsdkros.request.v20150901 import DescribeResourceDetailRequest

import ros.apps.config as connect
import ros.apps.utils as utils
import json
import sys

reload(sys)
sys.setdefaultencoding('utf-8')


def setup(subparsers):
    parser = subparsers.add_parser('describe-resource', help='Returns a description of the specified resource in the specified stack')
    parser.add_argument('--stack-name', help='The name of stack', required=True)
    parser.add_argument('--stack-id', help='The id of stack', required=True)
    parser.add_argument('--resource-name', help='The name of resource', required=True)

    parser.set_defaults(func=describe_resource)


def describe_resource(args):
    req = prepare_request(args)
    status, headers, body = utils.send_req(req)
    utils.deal_resp(status, headers, body, print_response)


def prepare_request(args):
    req = DescribeResourceDetailRequest.DescribeResourceDetailRequest()
    req.set_StackName(args.stack_name)
    req.set_StackId(args.stack_id)
    req.set_ResourceName(args.resource_name)

    if args.region_id is not None:
        req.set_headers({'x-acs-region-id': args.region_id})
    else:
        req.set_headers({'x-acs-region-id': connect.REGION_ID})

    return req


def print_response(data):
    if connect.JSON_FORM:
        jsonDumpsIndentStr = json.dumps(data, indent=connect.JSON_INDENT, ensure_ascii=False, sort_keys=True)
        print(jsonDumpsIndentStr)
    else:
        print('\n%-20s:  %s' % ('Id', data.get('Id')))
        print('%-20s:  %s' % ('Name', data.get('Name')))
        print('%-20s:  %s' % ('Type', data.get('Type')))
        print('%-20s:  %s' % ('Status', data.get('Status')))
        print('%-20s:  %s' % ('StatusReason', data.get('StatusReason')))
        print('%-20s:  %s' % ('ResourceData', data.get('ResourceData')))
        print('%-20s:  %s' % ('PhysicalId', data.get('PhysicalId')))
        print('%-20s:  %s' % ('Created', data.get('Created')))
        print('%-20s:  %s' % ('Updated', data.get('Updated')))
        print('%-20s:  %s\n' % ('Deleted', data.get('Deleted')))




# coding=utf-8

from setuptools import setup, find_packages

def readme():
    with open('README.rst') as f:
        return f.read()

setup (
    name = 'aliyun-ros-cli',
    version = '1.0.2',
    keywords = ('aliyun', 'ros', 'template', 'orchestration'),
    install_requires = ['aliyun-python-sdk-ros'],
    python_requires='>=2.6, <3',
    author = 'shenggong.wang, quming.ly',
    author_email = 'shenggong.wang@alibaba-inc.com',
    scripts=['bin/ros'],
    packages = find_packages(),
    include_package_data=True,
    exclude_package_date={'':['.gitignore']},
    platforms = 'any',
    description = 'Aliyun ros command line tools.',
    long_description=readme()
)
